local item_io_lib = require 'stonehearth_ace.lib.item_io.item_io_lib'
local item_quality_lib = require 'stonehearth_ace.lib.item_quality.item_quality_lib'

local es_entities = {}

local log = radiant.log.create_logger('es_entities')

-- uris are key, value pairs of uri, quantity
-- quantity can also be a table of quality/quantity pairs
function es_entities.spawn_items(uris, origin, min_radius, max_radius, options, place_items)
   local items = {}
   options = options or {}
   local owner_id = options.owner
   owner_id = owner_id and type(owner_id) ~= 'string' and owner_id:get_player_id() or owner_id
   local quality = options.quality
   local quality_options = quality and {max_quality = owner_id and item_quality_lib.get_max_random_quality(owner_id) or nil}
   local inventory, universal_storage
   if owner_id and options.add_spilled_to_inventory then
      inventory = stonehearth.inventory:get_inventory(owner_id)
      if ethereal_storage.game_has_started then
         universal_storage = stonehearth_ace.universal_storage:get_universal_storage(owner_id)
      end
      --log:debug('spawn_items for owner %s with universal_storage %s', owner_id, tostring(universal_storage))
   end

   for uri, detail in pairs(uris) do
      local qualities
      if type(detail) == 'number' then
         qualities = {[1] = detail}
      else
         qualities = detail
      end
      for item_quality, quantity in pairs(qualities) do
         if quality_options and item_quality > 1 then
            quality_options.min_quality = item_quality
         end
         local this_quality = quality or item_quality

         for i = 1, quantity do
            -- log:debug('trying to create %s with options: %s', uri, radiant.util.table_tostring(options))
            local item = radiant.entities.create_entity(uri, options)
            item_quality_lib.apply_quality(item, this_quality, quality_options)

            items[item:get_id()] = item

            if place_items ~= false then
               if universal_storage then
                  if not universal_storage:add_component('stonehearth:storage'):add_item(item) then
                     -- item couldn't be added, so destroy it! no litter!
                     radiant.entities.destroy_entity(item)
                  end
               else
                  local location = radiant.terrain.find_placement_point(origin, min_radius, max_radius)
                  radiant.terrain.place_entity(item, location)

                  if inventory then
                     inventory:add_item_if_not_full(item)
                  end
               end
            end
         end
      end
   end

   return items
end

function es_entities.output_items(uris, origin, min_radius, max_radius, options) --, output, inputs, spill_fail_items, quality)
   options = options or {}
   local inputs = options.inputs
   local output = options.output
   local owner_id = options.owner
   owner_id = owner_id and type(owner_id) ~= 'string' and owner_id:get_player_id() or owner_id

   local output_comp = output and output:is_valid() and output:get_component('stonehearth_ace:output')
   if inputs and type(inputs) ~= 'table' then
      inputs = {[inputs:get_id()] = inputs}
   elseif not inputs then
      inputs = {}
   end

   -- if this is before loading has finished, don't use the universal storage
   -- the item can get dropped on the ground, and then it will later be put in storage
   if ethereal_storage.game_has_started then
      local universal_storage = owner_id and stonehearth_ace.universal_storage:get_universal_storage(owner_id)
      if universal_storage then
         inputs[universal_storage:get_id()] = universal_storage
      end
   end

   options.inputs = inputs

   if not output_comp and (not inputs or not next(inputs)) then
      if options.spill_fail_items then
         local result = radiant.entities.get_empty_output_table()
         result.spilled = radiant.entities.spawn_items(uris, origin, min_radius, max_radius, options, true)
         return result
      else
         return radiant.entities.get_empty_output_table()
      end
   end

   local items = radiant.entities.spawn_items(uris, origin, min_radius, max_radius, options, false)
   return radiant.entities.output_spawned_items(items, origin, min_radius, max_radius, options)
end

function es_entities.output_spawned_items(items, origin, min_radius, max_radius, options) --, output, inputs, spill_fail_items, delete_fail_items)
   --local output_comp = output and output:is_valid() and output:get_component('stonehearth_ace:output')
   options = options or {}
   local inputs = options.inputs
   local owner_id = options.owner
   owner_id = owner_id and type(owner_id) ~= 'string' and owner_id:get_player_id() or owner_id
   
   if inputs and type(inputs) ~= 'table' then
      inputs = {[inputs:get_id()] = inputs}
   elseif not inputs then
      inputs = {}
   end
   
   -- if this is before loading has finished, don't use the universal storage
   -- the item can get dropped on the ground, and then it will later be put in storage
   if ethereal_storage.game_has_started then
      local universal_storage = owner_id and stonehearth_ace.universal_storage:get_universal_storage(owner_id)
      if universal_storage then
         inputs[universal_storage:get_id()] = universal_storage
      end
   end

   options.inputs = inputs

   options.spill_origin = origin
   options.spill_min_radius = min_radius
   options.spill_max_radius = max_radius

   --log:debug('output_spawned_items for owner %s with universal_storage %s', owner_id, tostring(universal_storage))
   return item_io_lib.try_output(items, inputs, options)
end

return es_entities
