local StorageComponent = require 'stonehearth.components.storage.storage_component'
EsStorageComponent = class()

local log = radiant.log.create_logger('storage_component')
local STORAGE_TYPES = {
   ['crate'] = true,
   ['urn'] = true,
}

-- when loading a game with existing entities with storage components in their jsons,
-- the components will get recreated; so if we create the component on an entity
-- already in the world, we can immediately remove it
EsStorageComponent._es_old_create = StorageComponent.create
function EsStorageComponent:create()
   if self:_consider_adding_universal() then
      self._entity:remove_component('stonehearth:storage')
      return
   end

   self:_es_old_create()
end

EsStorageComponent._es_old_restore = StorageComponent.restore
function EsStorageComponent:restore()
   local result = self:_consider_adding_universal() or false
   --log:debug('%s es_restore: %s', self._entity, result)

   self:_es_old_restore()
end

EsStorageComponent._es_old__on_parent_changed = StorageComponent._on_parent_changed
function EsStorageComponent:_on_parent_changed()
   --log:debug('%s _on_parent_changed', self._entity)
   if self:_consider_adding_universal() then
      log:debug('%s successfully considered adding universal storage; removing storage', self._entity)
      self._sv.items = {}
      self._sv.num_items = 0
      self._entity:remove_component('stonehearth:storage')
      local inventory = stonehearth.inventory:get_inventory(self._entity:get_player_id())
      if inventory then
         inventory:remove_storage(self._entity:get_id())
      end
      return
   end

   self:_es_old__on_parent_changed()
end

function EsStorageComponent:_consider_adding_universal()
   local universal_uri = stonehearth_ace.universal_storage:get_universal_storage_uri()
   local uri = self._entity:get_uri()
   if uri ~= universal_uri and STORAGE_TYPES[self._type] and self:is_public() and not self:is_hidden() and not self._sv.is_single_filter then
      -- also make sure it isn't the starting default storage for the town
      -- local pop = stonehearth.population:get_population(self._entity:get_player_id())
      -- local starting_items_container = pop:get_game_options().starting_items_container
      -- if starting_items_container == uri or starting_items_container == self._entity then
      --    log:debug('not converting %s to universal storage: it\'s the starting container!', self._entity)
      --    return
      -- end

      local town = stonehearth.town:get_town(self._entity:get_player_id())

      -- also make sure it isn't an herbalist planter or [fuel] consumer or anything involving the input component
      -- if it's default storage, it had the input component applied to it
      -- some of these may already be classed as input crates, single filter, or private/hidden storage, but just in case
      if self._entity:get_component('stonehearth_ace:herbalist_planter') or self._entity:get_component('stonehearth_ace:consumer') or
            self._entity:get_component('stonehearth_ace:auto_craft') or self._entity:get_component('stonehearth_ace:auto_io') or
            (self._entity:get_component('stonehearth_ace:input') and not (town and town:is_default_storage(self._entity))) then
         return
      end

      log:debug('%s is an appropriate type to convert to universal storage; checking location', self._entity)
      if radiant.entities.get_world_grid_location(self._entity) then
         -- if it's a 'crate' type storage entity in the world and it's not the universal storage entity, add the universal storage component
         -- the universal storage system should take care of the rest
         --log:debug('%s adding universal storage', self._entity)
         self._entity:add_component('stonehearth_ace:universal_storage')
         -- make sure it's not a default storage
         
         if town then
            town:remove_default_storage(self._entity:get_id())
            self._entity:remove_component('stonehearth_ace:input')
         end

         local storage = stonehearth_ace.universal_storage:register_storage(self._entity)
         if storage then
            storage:add_component('stonehearth:unit_info'):set_display_name('i18n(ethereal_storage:entities.ethereal_storage.display_name)')
         end
         return true
      end
   end
end

return EsStorageComponent